<?php

namespace Drupal\acquia_search;

use Drupal\acquia_connector\Subscription;
use Drupal\Component\Datetime\TimeInterface;
use Drupal\Component\Serialization\Json;
use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Logger\LoggerChannelInterface;
use GuzzleHttp\Client;
use GuzzleHttp\Exception\RequestException;

/**
 * Search v3 API Client.
 *
 * This is the v3 api client, used to fetch cores from Solr 6 servers used
 * by certain customers with the legacy version of Acquia Search.
 *
 * @package Drupal\acquia_search
 */
class AcquiaSearchV3ApiClient {

  /**
   * Cache backend.
   *
   * @var \Drupal\Core\Cache\CacheBackendInterface
   */
  protected $cache;

  /**
   * The HTTP client to fetch the feed data with.
   *
   * @var \GuzzleHttp\Client
   */
  protected $client;

  /**
   * Acquia Search Logger Channel.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * HTTP headers.
   *
   * @var array
   */
  protected $headers = [
    'Content-Type' => 'application/json',
    'Accept' => 'application/json',
  ];

  /**
   * Time Service.
   *
   * @var \Drupal\Component\Datetime\TimeInterface
   */
  protected $time;

  /**
   * Acquia Subscription Service.
   *
   * @var \Drupal\acquia_connector\Subscription
   */
  protected $subscription;

  /**
   * AcquiaSearchApiClient constructor.
   *
   * @param \Drupal\Core\Logger\LoggerChannelInterface $logger_channel
   *   Logger Channel service.
   * @param \Drupal\acquia_connector\Subscription $subscription
   *   The Acquia subscription service.
   * @param \GuzzleHttp\Client $http_client
   *   HTTP client.
   * @param \Drupal\Core\Cache\CacheBackendInterface $cache
   *   Cache backend.
   */
  public function __construct(LoggerChannelInterface $logger_channel, Subscription $subscription, Client $http_client, CacheBackendInterface $cache,  TimeInterface $date_time) {
    $this->logger = $logger_channel;
    $this->subscription = $subscription;
    $this->client = $http_client;
    $this->time = $date_time;
    $this->cache = $cache;
  }

  /**
   * Helper function to fetch all search v3 indexes for given network_id.
   *
   * @return array|false
   *   Response array or FALSE
   */
  public function getSearchIndexes() {
    $network_id = $this->subscription->getSettings()->getIdentifier();

    $result = [];
    if ($cache = $this->cache->get('acquia_search.v3indexes')) {
      if (is_array($cache->data) && $cache->expire > time()) {
        return $cache->data;
      }
    }
    $indexes = $this->searchRequest('/index/network_id/get_all?network_id=' . $network_id);
    if (is_array($indexes)) {
      if (!empty($indexes)) {
        foreach ($indexes as $index) {
          $result[] = [
            'balancer' => $index['host'],
            'core_id' => $index['name'],
            'version' => 'v3',
          ];
        }
      }
      // Cache will be set in both cases, 1. when search v3 cores are found and
      // 2. when there are no search v3 cores but api is reachable.
      $this->cache->set('acquia_search.v3indexes', $result, time() + (24 * 60 * 60));
      return $result;
    }
    else {
      // When api is not reachable, cache it for 1 minute.
      $this->cache->set('acquia_search.v3keys', $result, time() + (60));
    }

    return FALSE;
  }

  /**
   * Fetch the search v3 index keys for given core_id and network_id.
   *
   * @param string $core_id
   *   Core id.
   * @param string $network_id
   *   Acquia identifier.
   *
   * @return array|bool|false
   *   Search v3 index keys.
   */
  public function getKeys($core_id) {
    $network_id = $this->subscription->getSettings()->getIdentifier();

    if ($cache = $this->cache->get('acquia_search.v3keys')) {
      if (!empty($cache->data) && $cache->expire > time()) {
        return $cache->data;
      }
    }

    $keys = $this->searchRequest('/index/key?index_name=' . $core_id . '&network_id=' . $network_id);
    if ($keys) {
      // Cache will be set in both cases, 1. when search v3 cores are found and
      // 2. when there are no search v3 cores but api is reachable.
      $this->cache->set('acquia_search.v3keys', $keys, time() + (24 * 60 * 60));
      return $keys;
    }
    else {
      // When api is not reachable, cache it for 1 minute.
      $this->cache->set('acquia_search.v3keys', $keys, time() + (60));
    }

    return FALSE;

  }

  /**
   * Create and send a request to search controller.
   *
   * @param string $path
   *   Path to call.
   *
   * @return array|false
   *   Response array or FALSE.
   */
  public function searchRequest($path) {
    // Get the Searchv3 Host and SearchV3 API Key from Connector Metadata.
    $subscription = $this->subscription->getSubscription(TRUE);

    $data = [
      'host' => $subscription['acquia_search']['v3']['host'],
      'headers' => [
        'x-api-key' => $subscription['uuid'],
      ],
    ];
    $uri = $data['host'] . $path;
    $options = [
      'headers' => $data['headers'],
      'body' => Json::encode($data),
    ];

    try {
      $response = $this->client->get($uri, $options);
      if (!$response) {
        throw new \Exception('Empty Response');
      }
      $stream_size = $response->getBody()->getSize();
      $data = Json::decode($response->getBody()->read($stream_size));
      $status_code = $response->getStatusCode();

      if ($status_code < 200 || $status_code > 299) {
        $this->logger->error("Couldn't connect to search v3 API: @message",
          ['@message' => $response->getReasonPhrase()]);
        return FALSE;
      }
      return $data;
    }
    catch (RequestException $e) {
      if ($e->getCode() == 401) {
        $this->logger->error("Couldn't connect to search v3 API:
          Received a 401 response from the API indicating that credentials are incorrect.
          Please validate your credentials. @message", ['@message' => $e->getMessage()]);
      }
      elseif ($e->getCode() == 404) {
        $this->logger->error("Couldn't connect to search v3 API:
          Received a 404 response from the API indicating that the api host is incorrect.
          Please validate your host. @message", ['@message' => $e->getMessage()]);
      }
      else {
        $this->logger->error("Couldn't connect to search v3 API: Please
        validate your api host and credentials. @message", ['@message' => $e->getMessage()]);
      }
    }
    catch (\Exception $e) {
      $this->logger->error("Couldn't connect to search v3 API: @message",
        ['@message' => $e->getMessage()]);
    }

    return FALSE;
  }

}
