<?php

namespace Drupal\acquia_search\Plugin\SolrConnector;

use Drupal\acquia_connector\Subscription;
use Drupal\acquia_search\Client\Adapter\TimeoutAwarePsr18Adapter;
use Drupal\acquia_search\EventSubscriber\SearchSubscriber;
use Drupal\acquia_search\PreferredCoreService;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Plugin\PluginFormInterface;
use Drupal\Core\Url;
use Drupal\search_api_solr\SolrConnector\SolrConnectorPluginBase;
use Drupal\search_api_solr\SolrConnectorInterface;
use Http\Adapter\Guzzle6\Client as Guzzle6Client;
use Psr\Log\LoggerAwareInterface;
use Psr\Log\LoggerAwareTrait;
use Solarium\Core\Client\Client;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Acquia Search Plugin for Search API Solr.
 *
 * @package Drupal\acquia_search\Plugin\SolrConnector
 *
 * @SolrConnector(
 *   id = "solr_acquia_connector",
 *   label = @Translation("Acquia"),
 *   description = @Translation("Index items using an Acquia Apache Solr search server.")
 * )
 */
class SearchApiSolrAcquiaConnector extends SolrConnectorPluginBase implements
  SolrConnectorInterface,
  PluginFormInterface,
  ContainerFactoryPluginInterface,
  LoggerAwareInterface {
  use LoggerAwareTrait;

  /**
   * Event subscriber.
   *
   * @var \Drupal\acquia_search\EventSubscriber\SearchSubscriber
   */
  protected $searchSubscriber;

  /**
   * Acquia Subscription.
   *
   * @var \Drupal\acquia_connector\Subscription
   */
  protected $subscription;

  /**
   * Preferred Core Service.
   *
   * @var \Drupal\acquia_search\PreferredCoreService
   */
  protected $preferredCoreService;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $plugin = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $plugin->searchSubscriber = $container->get('acquia_search.search_subscriber');
    $plugin->subscription = $container->get('acquia_connector.subscription');
    $plugin->preferredCoreService = $container->get('acquia_search.preferred_core');
    $plugin->setLogger($container->get('logger.factory')->get('acquia_search'));
    return $plugin;
  }

  /**
   * Returns platform-specific Solr configuration.
   *
   * @return array
   *   Acquia platform Solr configuration.
   */
  public static function getPlatformConfig() {
    $preferredCoreService = \Drupal::service('acquia_search.preferred_core');

    // If auto-switch feature is turned off - do not attempt to determine the
    // preferred core.
    if ($preferredCoreService->isDisabled()) {
      return [];
    }

    $data = [
      'scheme' => 'https',
      'host' => $preferredCoreService->getPreferredCoreHostname(),
      'port' => 443,
      'path' => '/',
      'core' => $preferredCoreService->getPreferredCoreId(),
      'overridden_by_acquia_search' => $preferredCoreService->getReadOnly() ? ACQUIA_SEARCH_AUTO_OVERRIDE_READ_ONLY : ACQUIA_SEARCH_OVERRIDE_AUTO_SET,
    ];
    return $data;
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return self::getPlatformConfig();
  }

  /**
   * {@inheritdoc}
   *
   * Acquia-specific: 'admin/info/system' path is protected by Acquia.
   * Use admin/system instead.
   */
  public function pingServer() {
    return $this->doPing(['handler' => 'admin/system'], 'core');
  }

  /**
   * {@inheritdoc}
   */
  public function getServerInfo($reset = FALSE) {
    $this->useTimeout();
    return $this->getDataFromHandler('admin/system', $reset);
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);
    unset($form['host']);
    unset($form['port']);
    unset($form['path']);
    unset($form['core']);
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state) {
    // Turn off connection check of parent class.
  }

  /**
   * {@inheritdoc}
   */
  protected function connect() {

    if ($this->solr) {
      return;
    }

    // Create a PSR-18 adapter instance, since Solarium's HTTP adapter is
    // incompatible with remote_stream_wrapper.
    // See https://www.drupal.org/project/acquia_search/issues/3209704
    // And https://www.drupal.org/project/acquia_search_solr/issues/3171407
    $httpClient = new Guzzle6Client();
    $adapter = new TimeoutAwarePsr18Adapter($httpClient);

    $this->solr = new Client($adapter, $this->eventDispatcher);

    $this->configuration['key'] = 'core';
    // Reiterate https as the scheme.
    $this->configuration['scheme'] = 'https';
    $this->configuration[self::QUERY_TIMEOUT] = $this->configuration['timeout'];

    $this->solr->createEndpoint($this->configuration, TRUE);
    $this->solr->registerPlugin('acquia_search_subscriber', $this->searchSubscriber);
  }

  /**
   * {@inheritdoc}
   */
  protected function getServerUri() {
    $this->connect();
    return $this->solr->getEndpoint()->getServerUri();
  }

  /**
   * {@inheritdoc}
   *
   * Avoid providing an valid Update query if module determines this server
   * should be locked down (as indicated by the overridden_by_acquia_search
   * server option).
   *
   * @throws \Exception
   *   If the Search API Server is currently in read-only mode.
   */
  public function getUpdateQuery() {
    $this->connect();
    $overridden = $this->solr->getEndpoint('core')->getOption('overridden_by_acquia_search');
    if ($overridden === ACQUIA_SEARCH_AUTO_OVERRIDE_READ_ONLY) {
      $message = 'The Search API Server serving this index is currently in read-only mode.';
      \Drupal::logger('acquia search')->error($message);
      throw new \Exception($message);
    }
    return $this->solr->createUpdate();
  }

  /**
   * {@inheritdoc}
   */
  public function getExtractQuery() {
    $this->connect();
    $query = $this->solr->createExtract();
    $query->setHandler('extract/tika');
    return $query;
  }

  /**
   * {@inheritdoc}
   */
  public function getMoreLikeThisQuery() {
    $this->connect();
    $query = $this->solr->createMoreLikeThis();
    $query->setHandler('select');
    $query->addParam('qt', 'mlt');
    return $query;
  }

  /**
   * {@inheritdoc}
   */
  public function viewSettings() {

    $uri = Url::fromUri('https://docs.acquia.com/acquia-search/', ['absolute' => TRUE]);
    $link = Link::fromTextAndUrl($this->t('Acquia Search'), $uri);
    $message = $this->t('Search is provided by @acquia_search.', ['@acquia_search' => $link->toString()]);

    $this->messenger()->addStatus($message);

    return parent::viewSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function reloadCore() {
    return FALSE;
  }

}
