<?php

namespace Drupal\acquia_search;

use Drupal\acquia_connector\Subscription;
use Drupal\acquia_search\Event\AcquiaPossibleCoresEvent;
use Drupal\Component\EventDispatcher\ContainerAwareEventDispatcher;

/**
 * Preferred Core Service.
 *
 * Fetches the available and preferred cores based on env and subscription.
 *
 * @package Drupal\acquia_search
 */
class PreferredCoreService {

  /**
   * Event Dispatcher Service.
   *
   * @var \Drupal\Component\EventDispatcher\ContainerAwareEventDispatcher
   */
  protected $dispatcher;

  /**
   * Preferred search core.
   *
   * @var array
   */
  protected $preferredCore;

  /**
   * Readonly Status of Acquia Search
   *
   * @var bool
   */
  protected $readonly;

  /**
   * @var \Drupal\acquia_connector\Subscription
   */
  protected $subscription;

  /**
   * @var \Drupal\acquia_search\AcquiaSearchV3ApiClient
   */
  protected $acquiaSearchApiClient;

  /**
   * Preferred Search Core Service constructor.
   *
   *   E.g.
   *     [
   *       [
   *         'balancer' => 'useast11-c4.acquia-search.com',
   *         'core_id' => 'WXYZ-12345.dev.mysitedev',
   *       ],
   *     ].
   *
   * @param \Drupal\Component\EventDispatcher\ContainerAwareEventDispatcher $dispatcher
   *   The event dispatcher.
   * @param \Drupal\acquia_connector\Subscription $subscription
   *   Acquia Subscription Service.
   * @param \Drupal\acquia_search\AcquiaSearchV3ApiClient $acquia_search_api_client
   *   Acquia Search API Client.
   *
   */
  public function __construct(ContainerAwareEventDispatcher $dispatcher, Subscription $subscription, AcquiaSearchV3ApiClient $acquia_search_api_client) {
    $this->dispatcher = $dispatcher;
    $this->subscription = $subscription;
    $this->acquiaSearchApiClient = $acquia_search_api_client;
  }

  /**
   * @return bool
   *   Preferred Core Service is disabled.
   */
  public function isDisabled() {
    $subscription = $this->subscription->getSubscription();
    return !empty($subscription['acquia_search']['disable_auto_switch']) ?? FALSE;
  }
  /**
   * Fetch applicable search host for subscription.
   *
   * Acquia Search has different search hosts depending on the subscription data
   * recieved.
   *
   * @return string
   *   Search server url.
   */
  public function getSearchHost() {
    $subscription = $this->subscription->getSubscription();

    // By default return the main acquia governor search host.
    $search_host = \Drupal::config('acquia_search.settings')->get('host');
    // Adding the subscription specific colony to the heartbeat data.
    if (!empty($subscription['heartbeat_data']['search_service_colony'])) {
      $search_host = $subscription['heartbeat_data']['search_service_colony'];
    }
    // Check if we are on Acquia Cloud hosting. @see NN-2503.
    if (!empty($_ENV['AH_SITE_ENVIRONMENT']) && !empty($_ENV['AH_CURRENT_REGION'])) {
      if ($_ENV['AH_CURRENT_REGION'] == 'us-east-1' && $search_host == 'search.acquia.com') {
        $search_host = 'internal-search.acquia.com';
      }
      elseif (strpos($search_host, 'search-' . $_ENV['AH_CURRENT_REGION']) === 0) {
        $search_host = 'internal-' . $search_host;
      }
    }
    return $search_host;
  }

  /**
   * Returns a formatted list of Available cores.
   *
   * @return array|bool|null
   */
  public function getListOfAvailableCores() {
    $cores = $this->getAvailableCores();
    // We use core id as a key.
    return array_keys($cores);
  }

  /**
   * Returns core IDs available in subscription.
   */
  public function getAvailableCores() {
    $cores = [];
    // Do not return any available cores if subscription is empty
    if (!$this->subscription->isActive()) {
      return [];
    }

    $subscription = $this->subscription->getSubscription();
    // Attempt to fetch v3 cores first.
    if (isset($subscription['acquia_search']['v3']['host'])) {
      if (!$cores = $this->acquiaSearchApiClient->getSearchIndexes()) {
        // Throw exception?
        return NULL;
      }
    }
    else if (!empty($subscription['heartbeat_data']['search_cores'])) {
      foreach ($subscription['heartbeat_data']['search_cores'] as $core) {
        $cores[$core['core_id']] = $core;
      }
    }

    // Able to connect, however no cores were found return empty instead.
    if (empty($cores)) {
      return [];
    }

    // We user core id as a key.
    return $cores;
  }

  /**
   * Returns expected core ID based on the current site configs.
   *
   * @return string
   *   Core ID.
   */
  public function getPreferredCoreId() {
    $core = $this->getPreferredCore();
    return $core['core_id'] ?? NULL;
  }

  /**
   * Returns expected core host based on the current site configs.
   *
   * @return string
   *   Hostname.
   */
  public function getPreferredCoreHostname() {
    $core = $this->getPreferredCore();
    return $core['balancer'] ?? NULL;
  }

  /**
   * Determines whether the expected core ID matches any available core IDs.
   *
   * The list of available core IDs is set by Acquia and comes within the
   * Acquia Subscription information.
   *
   * @return bool
   *   True if the expected core ID available to use with Acquia.
   */
  public function isPreferredCoreAvailable() {
    return (bool) $this->getPreferredCore();
  }

  /**
   * Returns the preferred core from the list of available cores.
   *
   * @return array|null
   *   NULL or
   *     [
   *       'balancer' => 'useast11-c4.acquia-search.com',
   *       'core_id' => 'WXYZ-12345.dev.mysitedev',
   *     ].
   */
  public function getPreferredCore(): ?array {
    if (!empty($this->preferredCore)) {
      return $this->preferredCore;
    }

    $possible_cores = $this->getListOfPossibleCores();
    $available_cores = $this->sortCores($this->getAvailableCores());

    foreach ($possible_cores as $possible_core) {
      foreach ($available_cores as $available_core) {
        if ($possible_core === $available_core['core_id']) {
          $this->preferredCore = $available_core;
          return $this->preferredCore;
        }
      }
    }
    return NULL;
  }

  /**
   * Sorts and returns search cores.
   *
   * It puts v3 cores first.
   *
   * @param array $cores
   *   Unsorted array of search cores.
   *
   * @return array
   *   Array of search cores. V3 cores in the begging of the result array.
   */
  protected function sortCores(array $cores) {

    $v3_cores = array_filter($cores, function ($core) {
      return (!empty($core['version']) && $core['version'] === 'v3');
    });

    $regular_cores = array_filter($cores, function ($core) {
      return empty($core['version']) || $core['version'] !== 'v3';
    });

    return array_merge($v3_cores, $regular_cores);
  }

  /**
   * Returns a list of all possible search core IDs.
   *
   * The core IDs are generated based on the current site configuration.
   *
   * @return array
   *   E.g.
   *     [
   *       'WXYZ-12345.dev.mysitedev_db',
   *       'WXYZ-12345.dev.mysitedev_folder1',
   *     ]
   */
  public function getListOfPossibleCores() {
    // Do not return any available cores if subscription is empty
    if (!$this->subscription->isActive()) {
      $this->readonly = TRUE;
      return [];
    }

    $possible_core_ids = [];
    $event = new AcquiaPossibleCoresEvent($possible_core_ids);
    $this->dispatcher->dispatch($event, AcquiaSearchEvents::GET_POSSIBLE_CORES);
    $this->readonly = $event->getReadOnly();
    return $event->getPossibleCores();
  }

  /**
   * Returns whether the core should be read only.
   *
   * @return bool
   */
  public function getReadOnly() {
    // Fetch possible cores if the readonly value hasn't been set.
    if ($this->readonly === NULL) {
      $core = $this->getPreferredCore();
    }
    return $this->readonly;
  }

}
